#ifndef	__ACR120_H__
#define	__ACR120_H__

/***********************************
 * ACR120.h -- Win32 Linux Version *
 ***********************************/

#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */

#define ACR120_DECL
#define ACR120_DLLAPI

#ifndef	NO_ACR120_TYPE_BYTE
typedef unsigned char	BYTE;
#endif
#ifndef	NO_ACR120_TYPE_UINT8
typedef	signed char		INT8;
#endif
#ifndef	NO_ACR120_TYPE_UINT16
typedef	signed short	INT16;
#endif
#ifndef	NO_ACR120_TYPE_UINT32
typedef signed long		INT32;
#endif
#ifndef	NO_ACR120_TYPE_INT8
typedef unsigned char	UINT8;
#endif
#ifndef	NO_ACR120_TYPE_INT16
typedef unsigned short	UINT16;
#endif
#ifndef	NO_ACR120_TYPE_INT32
typedef unsigned long	UINT32;
#endif

/****** Error Code ******/
enum
{
	ERR_ACR120_INTERNAL_UNEXPECTED						= -1000,

	ERR_ACR120_PORT_INVALID								= -2000,
	ERR_ACR120_PORT_OCCUPIED							= -2010,
	ERR_ACR120_HANDLE_INVALID							= -2020,
	ERR_ACR120_INCORRECT_PARAM							= -2030,

	ERR_ACR120_READER_NO_TAG							= -3000,
	ERR_ACR120_READER_READ_FAIL_AFTER_OP				= -3010,
	ERR_ACR120_READER_NO_VALUE_BLOCK					= -3020,
	ERR_ACR120_READER_OP_FAILURE						= -3030,
	ERR_ACR120_READER_UNKNOWN							= -3040,

	ERR_ACR120_READER_LOGIN_INVALID_STORED_KEY_FORMAT	= -4010,
	ERR_ACR120_READER_WRITE_READ_AFTER_WRITE_ERROR		= -4020,
	ERR_ACR120_READER_DEC_FAILURE_EMPTY					= -4030
};

/****** Reader Port for AC_Open ******/
enum
{
	ACR120_COM1	= 0,
	ACR120_COM2	= (ACR120_COM1 + 1),
	ACR120_COM3	= (ACR120_COM1 + 2),
	ACR120_COM4	= (ACR120_COM1 + 3),
	ACR120_COM5	= (ACR120_COM1 + 4),
	ACR120_COM6	= (ACR120_COM1 + 5),
	ACR120_COM7	= (ACR120_COM1 + 6),
	ACR120_COM8	= (ACR120_COM1 + 7)
};

enum
{
	ACR120_COM_BAUDRATE_9600	= 0,
	ACR120_COM_BAUDRATE_19200	= 1,
	ACR120_COM_BAUDRATE_38400	= 2,
	ACR120_COM_BAUDRATE_57600	= 3,
	ACR120_COM_BAUDRATE_115200	= 4		/* Verison 1.0.1.0 [Edward] */
};

/****** Key Type for AC_Login ******/
enum
{
	ACR120_LOGIN_KEYTYPE_AA			= 0,
	ACR120_LOGIN_KEYTYPE_BB			= (ACR120_LOGIN_KEYTYPE_AA + 1),
	ACR120_LOGIN_KEYTYPE_FF			= (ACR120_LOGIN_KEYTYPE_BB + 1),
	ACR120_LOGIN_KEYTYPE_STORED_A	= (ACR120_LOGIN_KEYTYPE_FF + 1),
	ACR120_LOGIN_KEYTYPE_STORED_B	= (ACR120_LOGIN_KEYTYPE_STORED_A + 1)
};

/* [Since Version - 1.0.0.9] */
/****** Mifare Tag Type ******/
enum
{
	TAG_MIFARE_LIGHT			= 0x01,
	TAG_MIFARE_1K				= 0x02,
	TAG_MIFARE_4K				= 0x03,
	TAG_MIFARE_DESFIRE			= 0x04,
	TAG_MIFARE_ULTRALIGHT		= 0x05,
	TAG_JCOP30					= 0x06,
	TAG_SHANGHAI_TRANSPORT		= 0x07,
	TAG_MPCOS_COMBI				= 0x08,
	TAG_ISO_TYPEB				= 0x80
};


#define	ACR120_SN_LEN_4		(4)
#define	ACR120_SN_LEN_7		(7)
#define	ACR120_SN_LEN		(10)
/* [Since Version - 1.0.0.9] */
#define	ACR120_DATA_LEN		(16)
#define	ACR120_KEY_LEN		(6)
#define ACR120_MAX_TAG_LIST	(17)

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Open(INT16	ReaderPort,
											INT16	BaudRate);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Close(INT16	hReader);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Reset(	INT16	hReader,
												UINT8	stationID);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Select(	INT16	hReader,
												UINT8	stationID,
												BOOL*	pHaveTag,
												UINT8*	pTAG,
												UINT8	pSN[ACR120_SN_LEN]);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Login(	INT16	hReader,
												UINT8	stationID,
												UINT8	sector,
												UINT8	keyType,
												int		storedNo,
												UINT8	pKey[ACR120_KEY_LEN]);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Read(
									INT16	hReader,
									UINT8	stationID,
									UINT8	block,
									UINT8	pBlockData[ACR120_DATA_LEN]);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_ReadValue(	INT16	hReader,
													UINT8	stationID,
													UINT8	block,
													INT32*	pValueData);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_ReadEEPROM(	INT16	hReader,
													UINT8	stationID,
													UINT8	reg,
													UINT8*	pEEPROMData);

/* [Since Version - 1.0.0.9] */
ACR120_DLLAPI INT16 ACR120_DECL	ACR120_ReadLowLevelRegister(	INT16	hReader,
																UINT8	stationID,
																UINT8	reg,
																UINT8*	pRegData);
/* [Since Version - 1.0.0.9] */

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Write(
									INT16	hReader,
									UINT8	stationID,
									UINT8	block,
									UINT8	pBlockData[ACR120_DATA_LEN]);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_WriteValue(	INT16	hReader,
													UINT8	stationID,
													UINT8	block,
													INT32	valueData);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_WriteEEPROM(	INT16	hReader,
													UINT8	stationID,
													UINT8	reg,
													UINT8	eepromData);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_WriteMasterKey(
									INT16	hReader,
									UINT8	stationID,
									UINT8	keyNo,
									UINT8	pKey[ACR120_KEY_LEN]);

/* [Since Version - 1.0.0.9] */
ACR120_DLLAPI INT16 ACR120_DECL	ACR120_WriteLowLevelRegister(	INT16	hReader,
																UINT8	stationID,
																UINT8	reg,
																UINT8	registerData);
/* [Since Version - 1.0.0.9] */

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Inc(	INT16	hReader,
											UINT8	stationID,
											UINT8	block,
											INT32	value,
											INT32*	pNewValue);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Dec(	INT16	hReader,
											UINT8	stationID,
											UINT8	block,
											INT32	value,
											INT32*	pNewValue);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Copy(INT16	hReader,
											UINT8	stationID,
											UINT8	srcBlock,
											UINT8	desBlock,
											INT32*	pNewValue);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_Power(	INT16	hReader,
												UINT8	stationID,
												BOOL	bOn);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_ReadUserPort(INT16	hReader,
													UINT8	stationID,
													UINT8*	pUserPortState);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_WriteUserPort(	INT16	hReader,
														UINT8	stationID,
														UINT8	userPortState);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_GetID(	INT16	hReader,
												UINT8*	pNumID,
												UINT8*	pStationID);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_ListTag(	INT16	hReader,
												UINT8	stationID,
												UINT8*	pNumTagFound,
												BOOL*	pHaveTag,
												UINT8*	pTAG,
												UINT8*	pSN);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_MultiTagSelect(
									INT16	hReader,
									UINT8	stationID,
									UINT8	pSN[ACR120_SN_LEN],
									BOOL*	pHaveTag,
									UINT8*	pTAG,
									UINT8	pResultSN[ACR120_SN_LEN]);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_TxDataTelegram(
									INT16	hReader,
									UINT8	stationID,
									UINT8	length,
									BOOL	bParity,
									BOOL	bEvenParity,
									BOOL	bCRCGen,
									BOOL	bCRCCheck,
									BOOL	bCryptoInactive,
									UINT8	bitFrame,
									UINT8*	data,
									UINT8*	pRecvLen,
									UINT8*	recvData);

/* [Since Version - 1.0.0.9] */
ACR120_DLLAPI INT16 ACR120_DECL	ACR120_RequestVersionInfo(	INT16	hReader,
															UINT8	stationID,
															UINT8*  pVersionInfoLen,
															UINT8*	pVersionInfo);

ACR120_DLLAPI INT16 ACR120_DECL	ACR120_RequestDLLVersion(	UINT8*  pVersionInfoLen,
															UINT8*	pVersionInfo);
/* [Since Version - 1.0.0.9] */

#ifdef	__cplusplus
}
#endif	/* __cplusplus */

#endif	/* __ACR120_H__ */
